@@ -1,5 +1,5 @@
 package Test::Expr;
-our $VERSION = '0.000005';
+our $VERSION = '0.000006';
 
 use 5.012; use warnings;
 use Keyword::Declare;
@@ -16,6 +16,10 @@ sub _trim {
 }
 
 my $PERL_VAR = qr{ ((?&PerlVariable)) $PPR::GRAMMAR }xms;
+my $PERL_LITERAL = qr{^ (?> (?&PerlString)
+                          | (?&PerlQuotelikeQR)
+                          | (?&PerlQuotelikeQW)
+                          | (?&PerlNumber)        ) $ $PPR::GRAMMAR }xms;
 
 sub import {
     my ($package) = @_;
@@ -27,26 +31,59 @@ sub import {
 
     keyword ok (ListElem $test, Comma, ListElem $desc) {
         # Work out what values to report if there's a problem...
+        (my $test_code = $test) =~ s/^\s*do\s*\{(.*)\}\s*$/$1/;
+        my ($lhs, $op, $rhs)
+            = $test_code =~ m{(.*?)
+                              (<|(?!<-)>|<=|>=|==|!=|<=>|~~|\b(?:lt|gt|le|ge|eq|ne|cmp)\b)
+                              (.*)
+                              }xms;
+        my $test_setup;
+        my $test_expr;
+
+        if ($op) {
+            $test_setup = qq{my \$lhs = $lhs; my \$rhs = $rhs;};
+            $test_expr  = '$lhs '.$op.' $rhs';
+        }
+        else {
+            $lhs = $test_code;
+            $rhs = "";
+            $test_setup = qq{my \$lhs = $lhs;};
+            $test_expr = q{$lhs};
+        }
+
         my @vars = grep {defined} $test =~ m{$PERL_VAR}g;
-        my $var_len = max map {length} @vars;
-        my %seen;
-        my @diagnostics
-            = map {qq{diag sprintf(q{    %${var_len}s --> }, q{$_}), Data::Dump::dump($_);}}
+        ($lhs, $rhs, @vars) = map {_trim $_} $lhs, $rhs, @vars;
+        my $var_len = max map {length} $lhs, $rhs, @vars;
+        my %seen = ( $rhs => 1, $lhs => 1 );
+        my @diagnostics = (
+              ( $lhs !~ $PERL_LITERAL
+                  ? qq{diag sprintf(q{    %${var_len}s --> }, '$lhs'), Data::Dump::dump(\$lhs);}
+                  : ()
+              ),
+              ( $rhs && $rhs !~ $PERL_LITERAL
+                  ? qq{diag sprintf(q{    %${var_len}s --> }, '$rhs'), Data::Dump::dump(\$rhs);}
+                  : ()
+              ),
+              map {qq{diag sprintf(q{    %${var_len}s --> }, q{$_}), Data::Dump::dump($_);}}
                   grep { !$seen{$_}++ }
-                       @vars;
+                       @vars
+        );
 
         unshift @diagnostics, qq{diag q{  because:};}
             if @diagnostics;
-        unshift @diagnostics, qq{diag q{}; diag q{  ($test) was false};}
-            if (eval($desc)//'') ne qq{$test};
+        unshift @diagnostics, qq{diag q{}; diag q{  ($test_code) was false};}
+            if (eval($desc)//'') ne qq{$test_code};
 
         # Build the test code...
         qq{
-            if ($test) {
-                Test::More::ok(1, $desc);
-            }
-            else {
-                fail($desc); @diagnostics diag q{};
+            {
+                $test_setup
+                if ($test_expr) {
+                    Test::More::ok(1, $desc);
+                }
+                else {
+                    fail($desc); @diagnostics diag q{};
+                }
             }
         };
     }
@@ -62,7 +99,7 @@ Test::Expr - Test an expression with better error messages
 
 =head1 VERSION
 
-This document describes Test::Expr version 0.000005
+This document describes Test::Expr version 0.000006
 
 
 =head1 SYNOPSIS
